@@ -135,6 +135,11 @@ while ( $jud = $res->next() ) {
 	auditlog('judging', $jud['judgingid'], 'given back', null, $myhost);
 }
 
+// Warn when chroot has been disabled. This has security implications.
+if ( ! USE_CHROOT ) {
+	logmsg(LOG_WARNING, "Chroot disabled. This reduces judgehost security.");
+}
+
 // Create directory where to test submissions
 $workdirpath = JUDGEDIR . "/$myhost";
 system("mkdir -p $workdirpath/testcase", $retval);
