@@ -2,8 +2,6 @@
 
 # Script to test (run and compare) submissions with a single testcase
 #
-# $Id: testcase_run.sh 3543 2011-06-02 23:26:05Z eldering $
-
 # Usage: $0 <testdata.in> <testdata.out> <timelimit> <workdir>
 #           [<special-run> [<special-compare>]]
 #
@@ -37,7 +35,7 @@ cleanup ()
 {
 	# Remove some copied files to save disk space
 	if [ "$WORKDIR" ]; then
-		rm -f "$WORKDIR/dev/null" "$WORKDIR/bin/sh" "$WORKDIR/bin/runpipe"
+		rm -f "$WORKDIR/../dev/null" "$WORKDIR/../bin/sh" "$WORKDIR/../bin/runpipe"
 
 		# Replace testdata by symlinks to reduce disk usage
 		if [ -f "$WORKDIR/testdata.in" ]; then
@@ -90,7 +88,7 @@ fi
 # Location of scripts/programs:
 SCRIPTDIR="$DJ_LIBJUDGEDIR"
 STATICSHELL="$DJ_LIBJUDGEDIR/sh-static"
-GAINROOT="sudo"
+GAINROOT="sudo -n"
 RUNGUARD="$DJ_BINDIR/runguard"
 RUNPIPE="$DJ_BINDIR/runpipe"
 PROGRAM="execdir/program"
@@ -131,7 +129,7 @@ if [ -z "$USE_CHROOT" ] || [ "$USE_CHROOT" -eq 0 ]; then
 	unset USE_CHROOT
 	PREFIX=$PWD
 else
-	PREFIX=''
+	PREFIX="/`basename $PWD`"
 fi
 
 # Make testing/execute dir accessible for RUNUSER:
@@ -144,59 +142,42 @@ touch result.xml result.out      # Result of comparison (XML and plaintext versi
 touch program.out program.err    # Program output and stderr (for extra information)
 touch program.time program.exit  # Program runtime and exitcode
 
-# program.{out,err,time,exit} are written to by processes running as RUNUSER:
-chmod a+rw program.out program.err program.time program.exit
-
 logmsg $LOG_INFO "setting up testing (chroot) environment"
 
 # Copy the testdata input
 cd "$OLDDIR"
 cp "$TESTIN" "$WORKDIR/testdata.in"
 cd "$WORKDIR"
-chmod a+r testdata.in
 
-mkdir -m 0711 bin dev
+mkdir -p -m 0711 ../bin ../dev
 # Copy the run-script and a statically compiled shell:
 cp -p  "$RUN_SCRIPT"  ./run
-cp -pL "$STATICSHELL" ./bin/sh
-chmod a+rx run bin/sh
+cp -pL "$STATICSHELL" ../bin/sh
+chmod a+rx run ../bin/sh
 # If using a custom run script, copy additional support programs
 # if required:
 if [ -n "$SPECIALRUN" -a -f "$RUN_JURYPROG" ]; then
 	cp -p "$RUN_JURYPROG" ./runjury
-	cp -pL "$RUNPIPE"     ./bin/runpipe
-	chmod a+rx runjury bin/runpipe
+	cp -pL "$RUNPIPE"     ../bin/runpipe
+	chmod a+rx runjury ../bin/runpipe
 fi
 
 # We copy /dev/null: mknod (and the major/minor device numbers) are
 # not portable, while a fifo link has the problem that a cat program
 # must be run and killed.
 logmsg $LOG_DEBUG "creating /dev/null character-special device"
-$GAINROOT cp -pR /dev/null ./dev/null
-
-# Execute an optional chroot setup script:
-if [ "$USE_CHROOT" -a "$CHROOT_SCRIPT" ]; then
-	logmsg $LOG_DEBUG "executing chroot script: '$CHROOT_SCRIPT start'"
-	"$SCRIPTDIR/$CHROOT_SCRIPT" start
-fi
+$GAINROOT cp -pR /dev/null ../dev/null
 
 # Run the solution program (within a restricted environment):
 logmsg $LOG_INFO "running program (USE_CHROOT = ${USE_CHROOT:-0})"
 
-runcheck $GAINROOT $RUNGUARD ${DEBUG:+-v} ${USE_CHROOT:+-r "$PWD"} -u "$RUNUSER" \
-	-t $TIMELIMIT -m $MEMLIMIT -f $FILELIMIT -p $PROCLIMIT -c -o program.time -- \
-	$PREFIX/run $PREFIX/$PROGRAM \
-	testdata.in program.out program.err program.exit \
-	>error.tmp 2>&1
+runcheck ./run testdata.in program.out \
+	$GAINROOT $RUNGUARD ${DEBUG:+-v} ${USE_CHROOT:+-r "$PWD/.."} -u "$RUNUSER" \
+	-C $TIMELIMIT -t $((2*TIMELIMIT)) -m $MEMLIMIT -f $FILELIMIT -p $PROCLIMIT \
+	-c -s $FILELIMIT -e program.err -E program.exit -T program.time -- \
+	$PREFIX/$PROGRAM 2>error.tmp
 
-# Execute an optional chroot destroy script:
-if [ "$USE_CHROOT" -a "$CHROOT_SCRIPT" ]; then
-	logmsg $LOG_DEBUG "executing chroot script: '$CHROOT_SCRIPT stop'"
-	"$SCRIPTDIR/$CHROOT_SCRIPT" stop
-fi
-
-# Check for still running processes (first wait for all exiting processes):
-sleep 1
+# Check for still running processes:
 if ps -u "$RUNUSER" >/dev/null 2>&1 ; then
 	error "found processes still running"
 fi
@@ -214,7 +195,7 @@ fi
 
 # Check for errors from running the program:
 logmsg $LOG_DEBUG "checking program run exit-status"
-if grep  'timelimit reached: aborting command' error.tmp >/dev/null 2>&1 ; then
+if grep  'timelimit exceeded' error.tmp >/dev/null 2>&1 ; then
 	echo "Timelimit exceeded." >>error.out
 	cat error.tmp >>error.out
 	cleanexit ${E_TIMELIMIT:--1}
@@ -229,10 +210,6 @@ if [ "`cat program.exit`" != "0" ]; then
 	cat error.tmp >>error.out
 	cleanexit ${E_RUN_ERROR:--1}
 fi
-if [ $exitcode -ne 0 ]; then
-	cat error.tmp >>error.out
-	error "exitcode $exitcode without program.exit != 0"
-fi
 
 ############################################################
 ### Checks for other runtime errors:                     ###
