@@ -2,8 +2,6 @@
 /**
  * Common functions shared between team/public/jury interface
  *
- * $Id: common.php 3507 2010-12-16 20:06:14Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -11,6 +9,49 @@
 /** Text symbol used in output to represent a circle */
 define('CIRCLE_SYM', '&#9679;');
 
+function parseRunDiff($difftext){
+	$line = strtok($difftext,"\n"); //first line
+	if(sscanf($line, "### DIFFERENCES FROM LINE %d ###\n", $firstdiff) != 1)
+		return htmlspecialchars($difftext);
+	$return = $line . "\n";
+
+	// Add second line 'team ? reference'
+	$line = strtok("\n");
+	$return .= $line . "\n";
+
+	// We determine the line number width from the '_' characters and
+	// the separator position from the character '?' on the second line.
+	$linenowidth = strrpos($line, '_') + 1;
+	$midloc = strpos($line, '?') - ($linenowidth+1);
+
+	$line = strtok("\n");
+	while(strlen($line) != 0){
+		$linenostr = substr($line, 0, $linenowidth);
+		$diffline = substr($line, $linenowidth+1);
+		$mid = substr($diffline, $midloc-1, 3);
+		switch($mid){
+			case ' = ':
+				$formdiffline = "<span class='correct'>".htmlspecialchars($diffline)."</span>";
+				break;
+			case ' ! ':
+				$formdiffline = "<span class='differ'>".htmlspecialchars($diffline)."</span>";
+				break;
+			case ' $ ':
+				$formdiffline = "<span class='endline'>".htmlspecialchars($diffline)."</span>";
+				break;
+			case ' > ':
+			case ' < ':
+				$formdiffline = "<span class='extra'>".htmlspecialchars($diffline)."</span>";
+				break;
+			default:
+				$formdiffline = htmlspecialchars($diffline);
+		}
+		$return = $return . $linenostr . " " . $formdiffline . "\n";
+		$line = strtok("\n");
+	}
+	return $return;
+}
+
 /**
  * Print a list of submissions, either all or only those that
  * match <key> = <value>. Output is always limited to the
@@ -35,6 +76,13 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 			$verifyclause = '(j.verified = 0 OR s.judgehost IS NULL) ';
 		}
 	}
+	if ( isset($restrictions['judged']) ) {
+		if ( $restrictions['judged'] ) {
+			$judgedclause = '(j.result IS NOT NULL) ';
+		} else {
+			$judgedclause = '(j.result IS NULL) ';
+		}
+	}
 
 	$sqlbody =
 		'FROM submission s
@@ -51,9 +99,10 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 	$res = $DB->q('SELECT s.submitid, s.teamid, s.probid, s.langid,
 					s.submittime, s.judgehost, s.valid, t.name AS teamname,
 					p.name AS probname, l.name AS langname,
-					j.result, j.judgehost, j.verified, j.jury_member '
+					j.result, j.judgehost, j.verified, j.jury_member, j.seen '
 				  . $sqlbody
 				  . (isset($restrictions['verified'])  ? 'AND ' . $verifyclause : '')
+				  . (isset($restrictions['judged'])  ? 'AND ' . $judgedclause : '')
 				  .'ORDER BY s.submittime DESC, s.submitid DESC '
 				  . ($limit > 0 ? 'LIMIT 0, %i' : '%_')
 				, $cid, @$restrictions['teamid'], @$restrictions['probid']
@@ -71,23 +120,21 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 	}
 
 	// print the table with the submissions.
-	// table header; leave out the field that is our key (because it's the same
-	// for all rows)
+	// table header
 	echo "<table class=\"list sortable\">\n<thead>\n<tr>" .
 
-		(IS_JURY ? "<th scope=\"col\">ID</th>" : '') .
+		(IS_JURY ? "<th scope=\"col\" class=\"sorttable_numeric\">ID</th>" : '') .
 		"<th scope=\"col\">time</th>" .
 		(IS_JURY ? "<th scope=\"col\">team</th>" : '') .
 		"<th scope=\"col\">problem</th>" .
 		"<th scope=\"col\">lang</th>" .
 		"<th scope=\"col\">result</th>" .
-		(IS_JURY ? "<th scope=\"col\">judgehost</th>" : '') .
 		(IS_JURY ? "<th scope=\"col\">verified</th><th scope=\"col\">by</th>" : '') .
 
 		"</tr>\n</thead>\n<tbody>\n";
 
 	// print each row with links to detailed information
-	$iseven = $subcnt = $corcnt = $igncnt = $vercnt = 0;
+	$iseven = $subcnt = $corcnt = $igncnt = $vercnt = $quecnt = 0;
 	while( $row = $res->next() ) {
 
 		$sid = (int)$row['submitid'];
@@ -96,7 +143,8 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 		// present and valid.
 		if ( IS_JURY ) {
 			$link = ' href="submission.php?id=' . $sid . '"';
-		} elseif ( $row['result'] && $row['valid'] ) {
+		} elseif ( $row['result'] && $row['valid'] &&
+		           (!dbconfig_get('verification_required',0) || $row['verified']) ) {
 			$link = ' href="submission_details.php?id=' . $sid . '"';
 		} else {
 			$link = '';
@@ -115,6 +163,9 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 		if ( $sid == $highlight ) {
 			echo ' highlight';
 		}
+		if (!IS_JURY && !$row['seen'] ) {
+			echo ' unseen';
+		}
 		echo '">';
 
 		if ( IS_JURY ) {
@@ -145,7 +196,7 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 			echo '</a>';
 		} else {
 			if ( ! $row['result'] ||
-			     ( VERIFICATION_REQUIRED && ! $row['verified'] ) ) {
+			     ( dbconfig_get('verification_required', 0) && ! $row['verified'] ) ) {
 				if ( $row['submittime'] >= $cdata['endtime'] ) {
 					echo "<a>" . printresult('too-late') . "</a>";
 				} else {
@@ -159,15 +210,6 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 		echo "</td>";
 
 		if ( IS_JURY ) {
-			echo "<td><a$link>";
-			$judgehost = $row['judgehost'];
-			if ( !empty($judgehost) ) {
-					echo printhost($judgehost);
-			} else {
-				echo '&nbsp;';
-			}
-			echo '</a></td>';
-
 			// only display verification if we're done with judging
 			unset($verified, $jury_member);
 			$claim = FALSE;
@@ -233,10 +275,16 @@ function putSubmissions($cdata, $restrictions, $limit = 0, $highlight = null)
 					, $cid, @$restrictions['teamid'], @$restrictions['probid']
 					, @$restrictions['langid'], @$restrictions['judgehost']
 					);
+		$quecnt = $DB->q('VALUE SELECT count(s.submitid) ' . $sqlbody
+						.' AND result IS NULL'
+					, $cid, @$restrictions['teamid'], @$restrictions['probid']
+					, @$restrictions['langid'], @$restrictions['judgehost']
+					);
 		}
 		echo "<p>Total correct: $corcnt, submitted: $subcnt";
 		if($vercnt > 0)	echo ", unverified: $vercnt";
 		if($igncnt > 0) echo ", ignored: $igncnt";
+		if($quecnt > 0) echo ", judgement pending: $quecnt";
 		echo "</p>\n\n";
 	}
 
@@ -303,7 +351,7 @@ function putTeam($login) {
 	}
 
 	if ( !empty($team['room']) ) {
-		echo '<tr><td scope="row">Room:</td><td>' .
+		echo '<tr><td scope="row">Location:</td><td>' .
 			htmlspecialchars($team['room']) . "</td></tr>\n";
 	}
 
@@ -380,11 +428,11 @@ function checkFileUpload($errorcode) {
 			error('The uploaded file was only partially uploaded.');
 		case UPLOAD_ERR_NO_FILE:
 			error('No file was uploaded.');
-		case 6:	// UPLOAD_ERR_NO_TMP_DIR, constant doesn't exist in our minimal PHP version
+		case UPLOAD_ERR_NO_TMP_DIR:
 			error('Missing a temporary folder. Contact staff.');
-		case 7: // UPLOAD_ERR_CANT_WRITE
+		case UPLOAD_ERR_CANT_WRITE:
 			error('Failed to write file to disk. Contact staff.');
-		case 8: // UPLOAD_ERR_EXTENSION
+		case UPLOAD_ERR_EXTENSION:
 			error('File upload stopped by extension. Contact staff.');
 		default:
 			error('Unknown error while uploading: '. $_FILES['code']['error'] .
