@@ -3,8 +3,6 @@
 /**
  * Functions for calculating the scoreboard.
  *
- * $Id: scoreboard.php 3685 2011-10-11 15:22:37Z kink $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -22,13 +20,13 @@ require_once(LIBDIR . '/lib.misc.php');
  * This expects bool $solved (whether there was at least one correct
  * submission by this team for this problem) and int $num_submissions
  * (the total number of tries for this problem by this team)
- * as input, uses the constant PENALTY_TIME and outputs the number
+ * as input, uses the 'penalty_time' variable and outputs the number
  * of penalty minutes.
  *
  * The current formula is as follows:
  * - Penalty time is only counted for problems that the team finally
  *   solved. Yet unsolved problems always have zero penalty minutes.
- * - The penalty is PENALTY_TIME (usually 20 minutes) for each
+ * - The penalty is 'penalty_time' (usually 20 minutes) for each
  *   unsuccessful try. By definition, the number of unsuccessful
  *   tries is the number of submissions for a problem minus 1: the
  *   final, correct one.
@@ -38,7 +36,7 @@ function calcPenaltyTime($solved, $num_submissions)
 {
 	if ( ! $solved ) return 0;
 
-	return ( $num_submissions - 1 ) * PENALTY_TIME;
+	return ( $num_submissions - 1 ) * dbconfig_get('penalty_time', 20);
 }
 
 /**
@@ -56,10 +54,10 @@ function calcPenaltyTime($solved, $num_submissions)
  * scores[login](num_correct, total_time, solve_times[], rank,
  *               teamname, categoryid, sortorder, country, affilid)
  *
- * matrix[login][probid](is_correct, num_submissions, time, penalty)
+ * matrix[login][probid](is_correct, num_submissions, num_pending, time, penalty)
  *
- * summary(num_correct, total_time, affils[affilid], countries[country], problems[probid])
- *    probid(num_submissions, num_correct, best_time)
+ * summary(num_correct, total_time, affils[affilid], countries[country], problems[probid]
+ *    probid(num_submissions, num_pending, num_correct, best_time, best_time_sort[sortorder] )
  */
 function genScoreBoard($cdata, $jury = FALSE, $filter = NULL) {
 
@@ -83,24 +81,30 @@ function genScoreBoard($cdata, $jury = FALSE, $filter = NULL) {
 	// Don't leak information before start of contest
 	if ( ! $cstarted && ! $jury ) return;
 
-	// get the teams and problems
+	// get the teams, problems and categories
 	$teams = $DB->q('KEYTABLE SELECT login AS ARRAYKEY, login, team.name,
-	                 team.categoryid, team.affilid, country, sortorder
+	                 team.categoryid, team.affilid, sortorder,
+	                 country, color, team_affiliation.name AS affilname
 	                 FROM team
 	                 LEFT JOIN team_category
 	                        ON (team_category.categoryid = team.categoryid)
 	                 LEFT JOIN team_affiliation
 	                        ON (team_affiliation.affilid = team.affilid)
-	                 WHERE TRUE' .
+	                 WHERE enabled = 1' .
 	                ( $jury ? '' : ' AND visible = 1' ) .
 	                (isset($filter['affilid']) ? ' AND team.affilid IN (%As) ' : ' %_') .
 	                (isset($filter['country']) ? ' AND country IN (%As) ' : ' %_') .
 	                (isset($filter['categoryid']) ? ' AND team.categoryid IN (%As) ' : ' %_'),
 	                @$filter['affilid'], @$filter['country'], @$filter['categoryid']);
 
-	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY, probid FROM problem
+	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY,
+	                 probid, name, color FROM problem
 	                 WHERE cid = %i AND allow_submit = 1
 	                 ORDER BY probid', $cid);
+	$categs = $DB->q('KEYTABLE SELECT categoryid AS ARRAYKEY,
+ 	                  categoryid, name, color FROM team_category ' .
+	                 ($jury ? '' : 'WHERE visible = 1 ' ) .
+	                 'ORDER BY sortorder,name,categoryid');
 
 	// initialize the arrays we'll build from the data
 	$MATRIX = $SCORES = array();
@@ -133,6 +137,17 @@ function genScoreBoard($cdata, $jury = FALSE, $filter = NULL) {
 		$SCORES[$login]['country']     = $team['country'];
 	}
 
+	// initialize all problems with data
+	foreach( array_keys($probs) as $prob ) {
+		if ( !isset($SUMMARY['problems'][$prob]) ) {
+			$SUMMARY['problems'][$prob]['num_submissions'] = 0;
+			$SUMMARY['problems'][$prob]['num_pending'] = 0;
+			$SUMMARY['problems'][$prob]['num_correct'] = 0;
+			$SUMMARY['problems'][$prob]['best_time'] = NULL;
+			$SUMMARY['problems'][$prob]['best_time_sort'] = array();
+		}
+	}
+
 	// loop all info the scoreboard cache and put it in our own datastructure
 	while ( $srow = $scoredata->next() ) {
 
@@ -146,6 +161,7 @@ function genScoreBoard($cdata, $jury = FALSE, $filter = NULL) {
 		$MATRIX[$srow['teamid']][$srow['probid']] = array (
 			'is_correct'      => (bool) $srow['is_correct'],
 			'num_submissions' => $srow['submissions'],
+			'num_pending'     => $srow['pending'],
 			'time'            => $srow['totaltime'],
 			'penalty'         => $penalty );
 
@@ -189,53 +205,33 @@ function genScoreBoard($cdata, $jury = FALSE, $filter = NULL) {
 
 			// provide default scores when nothing submitted for this team,problem yet
 			if ( ! isset ( $MATRIX[$team][$prob] ) ) {
-				$MATRIX[$team][$prob] = array ( 'num_submissions' => 0, 'is_correct' => 0,
-				                                'time' => 0, 'penalty' => 0);
+				$MATRIX[$team][$prob] = array('num_submissions' => 0, 'num_pending' => 0,
+				                              'is_correct' => 0, 'time' => 0, 'penalty' => 0);
 			}
 			$pdata = $MATRIX[$team][$prob];
+			$psum = &$SUMMARY['problems'][$prob];
 
 			// update summary data for the bottom row
-			@$SUMMARY['problems'][$prob]['num_submissions'] += $pdata['num_submissions'];
-			@$SUMMARY['problems'][$prob]['num_correct'] += ($pdata['is_correct'] ? 1 : 0);
+			@$psum['num_submissions'] += $pdata['num_submissions'];
+			@$psum['num_pending'] += $pdata['num_pending'];
+			@$psum['num_correct'] += ($pdata['is_correct'] ? 1 : 0);
+
 			if ( $pdata['is_correct'] ) {
-				@$SUMMARY['problems'][$prob]['times'][] = $pdata['time'];
+				// store per sortorder the first solve time
+				if ( !isset($psum['best_time_sort'][$totals['sortorder']]) ||
+				     $pdata['time']<$psum['best_time_sort'][$totals['sortorder']] ) {
+					@$psum['best_time_sort'][$totals['sortorder']] = $pdata['time'];
+				}
+
+				// also keep overall best time per problem for in bottom summary row
+				if ( !isset($psum['best_time']) ||
+				     $pdata['time'] < @$psum['best_time'] ) {
+					@$psum['best_time'] = $pdata['time'];
+				}
 			}
 		}
 	}
 
-	// Fill all problems with data if not set already
-	foreach( array_keys($probs) as $prob ) {
-		if ( !isset($SUMMARY['problems'][$prob]) ) {
-			$SUMMARY['problems'][$prob]['num_submissions'] = 0;
-			$SUMMARY['problems'][$prob]['num_correct'] = 0;
-		}
-		if ( isset($SUMMARY['problems'][$prob]['times']) ) {
-			$SUMMARY['problems'][$prob]['best_time'] = min(@$SUMMARY['problems'][$prob]['times']);
-		} else {
-			$SUMMARY['problems'][$prob]['best_time'] = NULL;
-		}
-
-	}
-
-	// get the teams and problems
-	$teams = $DB->q('KEYTABLE SELECT login AS ARRAYKEY,
-	                 login, team.name, team.categoryid, team.affilid, sortorder,
-	                 color, country, team_affiliation.name AS affilname
-	                 FROM team
-	                 LEFT JOIN team_category
-	                        ON (team_category.categoryid = team.categoryid)
-	                 LEFT JOIN team_affiliation
-	                        ON (team_affiliation.affilid = team.affilid)' .
-	                ( $jury ? '' : ' WHERE visible = 1' ) );
-	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY,
-	                 probid, name, color FROM problem
-	                 WHERE cid = %i AND allow_submit = 1
-	                 ORDER BY probid', $cid);
-	$categs = $DB->q('KEYTABLE SELECT categoryid AS ARRAYKEY,
- 	                  categoryid, name, color FROM team_category ' .
-	                 ($jury ? '' : 'WHERE visible = 1 ' ) .
-	                 'ORDER BY sortorder,name,categoryid');
-
 	return array( 'matrix'     => $MATRIX,
 	              'scores'     => $SCORES,
 	              'summary'    => $SUMMARY,
@@ -276,6 +272,7 @@ function renderScoreBoardTable($cdata, $sdata, $myteamid = null,
 
 	// configuration
 	$SHOW_AFFILIATIONS = dbconfig_get('show_affiliations', 1);
+	$SHOW_PENDING      = dbconfig_get('show_pending', 0);
 
 	echo '<table class="scoreboard' . (IS_JURY ? ' scoreboard_jury' : '') . ($center ? ' center' : '') . "\">\n";
 
@@ -388,7 +385,11 @@ function renderScoreBoardTable($cdata, $sdata, $myteamid = null,
 			echo '<td class=';
 			// CSS class for correct/incorrect/neutral results
 			if( $matrix[$team][$prob]['is_correct'] ) {
-				echo '"score_correct"';
+				echo '"score_correct' .
+					( $summary['problems'][$prob]['best_time_sort'][$totals['sortorder']]==$matrix[$team][$prob]['time'] ?
+					  ' score_first' : '') . '"';
+			} elseif ( $matrix[$team][$prob]['num_pending'] > 0 && $SHOW_PENDING ) {
+				echo '"score_pending"';
 			} elseif ( $matrix[$team][$prob]['num_submissions'] > 0 ) {
 				echo '"score_incorrect"';
 			} else {
@@ -396,6 +397,10 @@ function renderScoreBoardTable($cdata, $sdata, $myteamid = null,
 			}
 			// number of submissions for this problem
 			$str = $matrix[$team][$prob]['num_submissions'];
+			// add pending submissions
+			if( $matrix[$team][$prob]['num_pending'] > 0 && $SHOW_PENDING ) {
+				$str .= ' + ' . $matrix[$team][$prob]['num_pending'];
+			}
 			// if correct, print time scored
 			if( $matrix[$team][$prob]['is_correct'] ) {
 				$str .= ' (' . $matrix[$team][$prob]['time'] . ' + ' .
@@ -411,7 +416,8 @@ function renderScoreBoardTable($cdata, $sdata, $myteamid = null,
 
 	if ( empty($limitteams) ) {
 		// print a summaryline
-		echo '<tbody><tr id="scoresummary" title="#submitted / #correct / fastest time">' .
+		echo '<tbody><tr id="scoresummary" title="#submitted' .
+		    ( $SHOW_PENDING ? ' + #pending' : '' ) . ' / #correct / fastest time">' .
 			'<td title="total teams">' .
 			jurylink(null,count($matrix)) . '</td>' .
 			( $SHOW_AFFILIATIONS ? '<td class="scoreaffil" title="#affiliations / #countries">' .
@@ -421,7 +427,9 @@ function renderScoreBoardTable($cdata, $sdata, $myteamid = null,
 			'<td title="total solved" class="scorenc">' . jurylink(null,$summary['num_correct'])  . '</td><td title=" "></td>';
 
 		foreach( array_keys($probs) as $prob ) {
-			$str = $summary['problems'][$prob]['num_submissions'] . ' / ' .
+			$str = $summary['problems'][$prob]['num_submissions'] .
+			       ( $SHOW_PENDING ? ' + ' .
+			         $summary['problems'][$prob]['num_pending'] : '' ) . ' / ' .
 			       $summary['problems'][$prob]['num_correct'] . ' / ' .
 				   ( isset($summary['problems'][$prob]['best_time']) ?
 					 $summary['problems'][$prob]['best_time'] : '-' );
@@ -452,7 +460,6 @@ function renderScoreBoardTable($cdata, $sdata, $myteamid = null,
 		echo "</tbody>\n</table>\n\n";
 	}
 
-
 	return;
 }
 
@@ -496,7 +503,7 @@ function putScoreBoard($cdata, $myteamid = NULL, $static = FALSE, $filter = FALS
 		if ( $fdata['showfrozen'] ) {
 			echo " (";
 			if ( IS_JURY ) {
-				echo "public scoreboard is ";
+				echo '<a href="../public/">the public scoreboard</a> is ';
 			}
 			echo "frozen since " . printtime($cdata['freezetime']) .")";
 		}
@@ -504,32 +511,22 @@ function putScoreBoard($cdata, $myteamid = NULL, $static = FALSE, $filter = FALS
 	}
 
 	if ( $filter!==FALSE ) {
-		
+
+		$affils = $DB->q('KEYTABLE SELECT affilid AS ARRAYKEY, name, country
+		                  FROM team_affiliation');
+		$categids = $DB->q('KEYVALUETABLE SELECT categoryid, name FROM team_category');
+
 		$affilids  = array();
 		$countries = array();
-		$categids  = array();
-		foreach( $sdata['teams'] as $team ) {
-			if ( !empty($team['affilid']) ) {
-				$affilids[]  = $team['affilid'];
-				$countries[] = $team['country'];
-			}
-			$categids[] = $team['categoryid'];
+		foreach( $affils as $id => $affil ) {
+			$affilids[$id]  = $affil['name'];
+			$countries[] = $affil['country'];
 		}
 
 		$countries = array_unique($countries);
 		sort($countries);
 
-		$affilids = empty($affilids) ? array() : 
-				$DB->q('KEYVALUETABLE SELECT affilid, name FROM team_affiliation
-		                    WHERE affilid IN (%As)', $affilids);
-		$categids = empty($categids) ? array() :
-				$DB->q('KEYVALUETABLE SELECT categoryid, name FROM team_category
-		                    WHERE categoryid IN (%As)', $categids);
-
-		if ( count($categids) > 1 || count($countries) > 1 || count($affilids) > 1 ) {
-
-			require_once(LIBWWWDIR . '/forms.php');
-
+		require_once(LIBWWWDIR . '/forms.php');
 		?>
 
 <table class="scorefilter">
@@ -540,7 +537,7 @@ function putScoreBoard($cdata, $myteamid = NULL, $static = FALSE, $filter = FALS
 
 		echo addForm($pagename, 'get') .
 			( count($affilids) > 1 ? addSelect('affilid[]',    $affilids,  @$filter['affilid'],    TRUE,  8) : "" ) .
-			( count($countries) > 1 ? addSelect('country[]',    $countries, @$filter['country'],    FALSE, 8) : "" ) .
+			( count($countries)> 1 ? addSelect('country[]',    $countries, @$filter['country'],    FALSE, 8) : "" ) .
 			( count($categids) > 1 ? addSelect('categoryid[]', $categids,  @$filter['categoryid'], TRUE,  8) : "" ) .
 			addSubmit('filter') . addSubmit('clear', 'clear') .
 			addEndForm();
@@ -553,7 +550,6 @@ collapse("filter");
 -->
 </script>
 		<?php
-		}
 	}
 
 	renderScoreBoardTable($cdata,$sdata,$myteamid,$static);
@@ -566,7 +562,8 @@ collapse("filter");
 		$lastupdate = time();
 	}
 	echo "<p id=\"lastmod\">Last Update: " .
-		date('j M Y H:i', $lastupdate) . "</p>\n\n";
+	     date('j M Y H:i', $lastupdate) . "<br />\n" .
+	     "using <a href=\"http://domjudge.sourceforge.net/\">DOMjudge</a></p>\n\n";
 
 	return;
 }
