@@ -194,7 +194,7 @@ if ( isset($jid) )  {
 		($jud['valid'] == 1 ? '' : ' (INVALID)') . "</h2>\n\n";
 
 	if ( !$jud['verified'] ) {
-		echo addForm($pagename . '?id=' . urlencode($id) . '&jid=' . urlencode($jid));
+		echo addForm($pagename . '?id=' . urlencode($id) . '&amp;jid=' . urlencode($jid));
 
 		echo "<p>Claimed: " .
 		    "<strong>" . printyn(!empty($jud['jury_member'])) . "</strong>";
@@ -224,7 +224,7 @@ if ( isset($jid) )  {
 				echo addForm('verify.php') .
 				    addHidden('id',  $jud['judgingid']) .
 				    addHidden('val', $val) .
-				    addHidden('redirect', $_SERVER['HTTP_REFERER']);
+				    addHidden('redirect', @$_SERVER['HTTP_REFERER']);
 			}
 
 			echo "<p>Verified: " .
