@@ -2,8 +2,6 @@
 /**
  * View a row in a team_category
  *
- * $Id: team_category.php 3467 2010-11-29 18:17:31Z kink $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -48,7 +46,7 @@ if ( IS_ADMIN && ($cmd == 'add' || $cmd == 'edit') ) {
 <td><?php echo addInputField('text','data[0][color]', @$row['color'],
 	'size="15" maxlength="25" class="color {required:false,adjust:false,hash:true,caps:false}"')?>
 <a target="_blank"
-href="http://www.w3schools.com/css/css_colornames.asp"><img
+href="http://www.w3schools.com/cssref/css_colornames.asp"><img
 src="../images/b_help.png" class="smallpicto" alt="?" /></a></td></tr>
 </td></tr>
 
@@ -119,9 +117,7 @@ if ( $teams->count() == 0 ) {
 	}
 	echo "</tbody>\n</table>\n\n";
 
-	echo "<p>";
 	putTeamRow($cdata,$listteams);
-	echo "</p>\n\n";
 }
 
 
